@@ -1,9 +1,6 @@
 use std::path::PathBuf;
 
-use clap::{arg, Parser};
-use clap_complete::Shell;
-
-const LSP_VERSION: &str = env!("CARGO_PKG_VERSION");
+use clap::{Parser, arg};
 
 pub const fn get_styles() -> clap::builder::Styles {
     clap::builder::Styles::styled()
@@ -50,7 +47,7 @@ pub const fn get_styles() -> clap::builder::Styles {
     about="CMake Lsp implementation based on Tower and Tree-sitter", 
     long_about = None,
     author = "Cris",
-    version=LSP_VERSION
+    version,
 )]
 pub enum NeocmakeCli {
     #[command(long_flag = "stdio", about = "run with stdio")]
@@ -81,12 +78,6 @@ pub enum NeocmakeCli {
         #[arg(value_name = "tojson", short = 'j')]
         tojson: bool,
     },
-    #[command(long_flag = "generate", about = "generate the completion")]
-    GenCompletions {
-        // If provided, outputs the completion file for given shell
-        #[arg(value_enum, required = true)]
-        shell: Shell,
-    },
 }
 
 #[test]
