@@ -6,22 +6,20 @@ use std::path::{Path, PathBuf};
 use std::sync::{Arc, LazyLock};
 
 use builtin::{BUILTIN_COMMAND, BUILTIN_MODULE, BUILTIN_VARIABLE};
-use tokio::fs;
 use tokio::sync::Mutex;
 use tower_lsp::lsp_types::{
-    self, CompletionItem, CompletionItemKind, CompletionResponse, Documentation, MessageType,
-    Position,
+    CompletionItem, CompletionItemKind, CompletionResponse, Documentation, MessageType, Position,
 };
 
 use crate::consts::TREESITTER_CMAKE_LANGUAGE;
-use crate::languageserver::BUFFERS_CACHE;
+use crate::languageserver::get_or_update_buffer_context;
 use crate::scansubs::TREE_MAP;
-use crate::utils::treehelper::{get_pos_type, PositionType, ToPoint};
+use crate::utils::treehelper::{PositionType, ToPoint, get_pos_type};
 use crate::utils::{
-    gen_module_pattern, include_is_module, remove_quotation_and_replace_placeholders,
-    DocumentNormalize, LineCommentTmp, CACHE_CMAKE_PACKAGES_WITHKEYS,
+    CACHE_CMAKE_PACKAGES_WITHKEYS, LineCommentTmp, gen_module_pattern, include_is_module,
+    remove_quotation_and_replace_placeholders,
 };
-use crate::{fileapi, CMakeNodeKinds};
+use crate::{CMakeNodeKinds, fileapi};
 
 pub type CompleteKV = HashMap<PathBuf, Vec<CompletionItem>>;
 
@@ -33,6 +31,27 @@ pub static COMPLETE_CACHE: LazyLock<Arc<Mutex<CompleteKV>>> =
 #[cfg(unix)]
 const PKG_IMPORT_TARGET: &str = "IMPORTED_TARGET";
 
+pub fn init_builtin_command() {
+    let _ = &*BUILTIN_COMMAND;
+}
+pub fn init_builtin_module() {
+    let _ = &*BUILTIN_MODULE;
+}
+
+pub fn init_builtin_variable() {
+    let _ = &*BUILTIN_VARIABLE;
+}
+
+pub fn init_system_modules() {
+    let _ = &*crate::utils::CMAKE_PACKAGES_WITHKEY;
+    let _ = &*crate::utils::CMAKE_PACKAGES;
+    #[cfg(unix)]
+    {
+        let _ = &*crate::utils::packagepkgconfig::PKG_CONFIG_PACKAGES;
+        let _ = &*crate::utils::packagepkgconfig::PKG_CONFIG_PACKAGES_WITHKEY;
+    }
+}
+
 pub fn rst_doc_read(doc: &str, filename: &str) -> Vec<CompletionItem> {
     doc.lines()
         .filter(|line| line.starts_with(".. command:: "))
@@ -50,10 +69,9 @@ pub fn rst_doc_read(doc: &str, filename: &str) -> Vec<CompletionItem> {
 }
 
 pub async fn update_cache<P: AsRef<Path>>(path: P, context: &str) -> Vec<CompletionItem> {
-    let context = context.normalize();
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(&context, None);
+    let thetree = parse.parse(context, None);
     let tree = thetree.unwrap();
     let Some(result_data) = getsubcomplete(
         tree.root_node(),
@@ -83,13 +101,7 @@ pub async fn get_cached_completion<P: AsRef<Path>>(path: P) -> Vec<CompletionIte
         let complete_cache = COMPLETE_CACHE.lock().await;
         if let Some(data) = complete_cache.get(parent) {
             completions.append(&mut data.clone());
-        } else if let Ok(context) = fs::read_to_string(parent).await {
-            let mut buffer_cache = BUFFERS_CACHE.lock().await;
-            buffer_cache.insert(
-                lsp_types::Url::from_file_path(parent).unwrap(),
-                context.clone(),
-            );
-            drop(buffer_cache);
+        } else if let Ok(context) = get_or_update_buffer_context(parent).await {
             drop(complete_cache);
             completions.append(&mut update_cache(parent, context.as_str()).await);
             path.clone_from(parent);
@@ -110,20 +122,19 @@ pub async fn getcomplete<P: AsRef<Path>>(
     find_cmake_in_package: bool,
 ) -> Option<CompletionResponse> {
     let local_path = local_path.as_ref();
-    let source = source.normalize();
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(&source, None);
+    let thetree = parse.parse(source, None);
     let tree = thetree.unwrap();
     let mut complete: Vec<CompletionItem> = vec![];
 
     let current_point = location.to_point();
-    let postype = get_pos_type(current_point, tree.root_node(), &source);
+    let postype = get_pos_type(current_point, tree.root_node(), source);
     match postype {
         PositionType::VarOrFun | PositionType::TargetLink | PositionType::TargetInclude => {
-            let mut cached_compeletion = get_cached_completion(local_path).await;
-            if !cached_compeletion.is_empty() {
-                complete.append(&mut cached_compeletion);
+            let mut cached_completion = get_cached_completion(local_path).await;
+            if !cached_completion.is_empty() {
+                complete.append(&mut cached_completion);
             }
             if let Some(mut cmake_cache) = fileapi::get_complete_data() {
                 complete.append(&mut cmake_cache);
@@ -160,9 +171,9 @@ pub async fn getcomplete<P: AsRef<Path>>(
             complete.append(&mut findpackage::PKGCONFIG_SOURCE.clone());
         }
         PositionType::Include => {
-            let mut cached_compeletion = get_cached_completion(local_path).await;
-            if !cached_compeletion.is_empty() {
-                complete.append(&mut cached_compeletion);
+            let mut cached_completion = get_cached_completion(local_path).await;
+            if !cached_completion.is_empty() {
+                complete.append(&mut cached_completion);
             }
             if let Some(mut cmake_cache) = fileapi::get_complete_data() {
                 complete.append(&mut cmake_cache);
@@ -386,7 +397,7 @@ fn getsubcomplete<P: AsRef<Path>>(
                     let child = child.child(2).unwrap();
                     let mut advancedwalk = child.walk();
                     for identifier in child.children(&mut advancedwalk) {
-                        if identifier.kind() == "argument"
+                        if identifier.kind() == CMakeNodeKinds::ARGUMENT
                             && identifier.start_position().row == identifier.end_position().row
                         {
                             let startx = identifier.start_position().column;
@@ -978,11 +989,11 @@ fn test_complete_win() {
 
     use tempfile::tempdir;
 
-    let file_info = "set(AB \"100\")\r\n# test hello \r\nfunction(bb)\r\nendfunction()".normalize();
+    let file_info = "set(AB \"100\")\r\n# test hello \r\nfunction(bb)\r\nendfunction()";
 
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(&file_info, None).unwrap();
+    let thetree = parse.parse(file_info, None).unwrap();
     let dir = tempdir().unwrap();
     let root_cmake = dir.path().join("CMakeList.txt");
     let mut file = File::create(&root_cmake).unwrap();
