@@ -2,32 +2,47 @@ mod config;
 #[cfg(test)]
 mod test;
 use std::collections::HashMap;
+use std::path::{Path, PathBuf};
+use std::process::exit;
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, LazyLock, RwLock};
 
 use tokio::sync::Mutex;
 use tower_lsp::jsonrpc::{Error as LspError, Result};
 use tower_lsp::lsp_types::*;
-use tower_lsp::{lsp_types, LanguageServer};
+use tower_lsp::{LanguageServer, lsp_types};
 use tree_sitter::Parser;
 
 use self::config::Config;
 use super::Backend;
+use crate::config::CMAKE_LINT_CONFIG;
 use crate::consts::TREESITTER_CMAKE_LANGUAGE;
 use crate::fileapi::DEFAULT_QUERY;
 use crate::formatting::getformat;
-use crate::gammar::{checkerror, ErrorInformation, LintConfigInfo};
+use crate::gammar::{ErrorInformation, LintConfigInfo, checkerror};
 use crate::semantic_token::LEGEND_TYPE;
-use crate::utils::{did_vcpkg_project, treehelper, VCPKG_LIBS, VCPKG_PREFIX};
+use crate::utils::treehelper::ToPosition;
+use crate::utils::{VCPKG_LIBS, VCPKG_PREFIX, did_vcpkg_project, treehelper};
 use crate::{
-    ast, complete, document_link, fileapi, filewatcher, hover, jump, scansubs, semantic_token,
-    utils,
+    BackendInitInfo, ast, complete, document_link, fileapi, filewatcher, hover, jump, quick_fix,
+    rename, scansubs, semantic_token, utils,
 };
-
-pub static BUFFERS_CACHE: LazyLock<Arc<Mutex<HashMap<lsp_types::Url, String>>>> =
+pub static BUFFERS_CACHE: LazyLock<Arc<Mutex<HashMap<lsp_types::Uri, String>>>> =
     LazyLock::new(|| Arc::new(Mutex::new(HashMap::new())));
 
-static CLIENT_CAPABILITIES: RwLock<Option<TextDocumentClientCapabilities>> = RwLock::new(None);
+pub async fn get_or_update_buffer_context<P: AsRef<Path>>(path: P) -> std::io::Result<String> {
+    let uri = lsp_types::Uri::from_file_path(&path).unwrap();
+    let mut buffer_cache = BUFFERS_CACHE.lock().await;
+    if let Some(context) = buffer_cache.get(&uri) {
+        return Ok(context.clone());
+    }
+    let context = tokio::fs::read_to_string(&path).await?;
+    buffer_cache.insert(uri, context.clone());
+    Ok(context)
+}
 
+static CLIENT_CAPABILITIES: RwLock<Option<TextDocumentClientCapabilities>> = RwLock::new(None);
+static ENABLE_SNIPPET: AtomicBool = AtomicBool::new(false);
 fn set_client_text_document(text_document: Option<TextDocumentClientCapabilities>) {
     let mut data = CLIENT_CAPABILITIES.write().unwrap();
     *data = text_document;
@@ -38,7 +53,14 @@ pub fn get_client_capabilities() -> Option<TextDocumentClientCapabilities> {
     data.clone()
 }
 
-pub fn client_support_snippet() -> bool {
+fn init_snippet_setting(use_snippet: bool) {
+    ENABLE_SNIPPET.store(use_snippet, Ordering::Relaxed);
+}
+
+pub fn to_use_snippet() -> bool {
+    if !ENABLE_SNIPPET.load(Ordering::Relaxed) {
+        return false;
+    }
     match get_client_capabilities() {
         Some(c) => c
             .completion
@@ -50,19 +72,30 @@ pub fn client_support_snippet() -> bool {
 }
 
 impl Backend {
-    async fn path_in_project(&self, path: &str) -> bool {
-        if self.root_path.lock().await.is_none() {
-            return true;
-        }
-
-        // NOTE: not enough good, but is ok
-        !path.starts_with("/usr")
+    fn root_path(&self) -> Option<&PathBuf> {
+        self.root_path.get_or_init(|| None).as_ref()
     }
-    async fn publish_diagnostics(&self, uri: Url, context: String, lint_info: LintConfigInfo) {
-        if !self.path_in_project(uri.path()).await {
-            return;
-        }
 
+    fn init_info(&self) -> &BackendInitInfo {
+        self.init_info
+            .get()
+            .expect("Should have been inited before")
+    }
+
+    fn path_in_project<P: AsRef<Path>>(&self, path: P) -> bool {
+        let Some(root_path) = self.root_path() else {
+            return true;
+        };
+
+        let Some(diff) = pathdiff::diff_paths(path, root_path) else {
+            return false;
+        };
+        use std::path::Component;
+        diff.components()
+            .all(|component| component != Component::ParentDir)
+    }
+
+    async fn publish_diagnostics(&self, uri: Uri, context: String, lint_info: LintConfigInfo) {
         let Ok(file_path) = uri.to_file_path() else {
             tracing::error!("Cannot transport {uri:?} to file_path");
             self.client
@@ -74,6 +107,10 @@ impl Backend {
             return;
         };
 
+        if !self.path_in_project(&file_path) {
+            return;
+        }
+
         let gammererror = checkerror(&file_path, &context, lint_info);
         if let Some(diagnoses) = gammererror {
             let mut pusheddiagnoses = vec![];
@@ -84,10 +121,8 @@ impl Backend {
                 severity,
             } in diagnoses.inner
             {
-                let pointx =
-                    lsp_types::Position::new(start_point.row as u32, start_point.column as u32);
-                let pointy =
-                    lsp_types::Position::new(end_point.row as u32, end_point.column as u32);
+                let pointx = start_point.to_position();
+                let pointy = end_point.to_position();
                 let range = Range {
                     start: pointx,
                     end: pointy,
@@ -119,7 +154,7 @@ impl Backend {
                 uri.clone(),
                 context.to_string(),
                 LintConfigInfo {
-                    use_lint: self.init_info.lock().await.enable_lint,
+                    use_lint: self.init_info().enable_lint,
                     use_extra_cmake_lint: true,
                 },
             )
@@ -128,7 +163,6 @@ impl Backend {
     }
 }
 
-#[tower_lsp::async_trait]
 impl LanguageServer for Backend {
     async fn initialize(&self, initial: InitializeParams) -> Result<InitializeResult> {
         let initial_config: Config = initial
@@ -136,15 +170,20 @@ impl LanguageServer for Backend {
             .and_then(|value| serde_json::from_value(value).unwrap_or(None))
             .unwrap_or_default();
 
+        init_snippet_setting(initial_config.use_snippets());
+
         let do_format = initial_config.is_format_enabled();
 
-        let find_cmake_in_package = initial_config.is_scan_cmake_in_package();
+        let scan_cmake_in_package = initial_config.is_scan_cmake_in_package();
 
-        let lint_enable = initial_config.is_lint_enabled();
+        let enable_lint = initial_config.is_lint_enabled();
 
-        let mut init_info = self.init_info.lock().await;
-        init_info.scan_cmake_in_package = find_cmake_in_package;
-        init_info.enable_lint = lint_enable;
+        self.init_info
+            .set(BackendInitInfo {
+                scan_cmake_in_package,
+                enable_lint,
+            })
+            .expect("here should be the first place to init the init_info");
 
         if let Some(workspace) = initial.capabilities.workspace {
             if let Some(watch_file) = workspace.did_change_watched_files {
@@ -152,10 +191,12 @@ impl LanguageServer for Backend {
                     watch_file.dynamic_registration,
                     watch_file.relative_pattern_support,
                 ) {
+                    // NOTE: I think it only contains one workspace
                     if let Some(ref top_path) = initial
-                        .root_uri
+                        .workspace_folders
                         .as_ref()
-                        .and_then(|uri| uri.to_file_path().ok())
+                        .and_then(|folders| folders.first())
+                        .and_then(|folder| folder.uri.to_file_path().ok())
                     {
                         let path = top_path.join("build").join("CMakeCache.txt");
                         if path.exists() {
@@ -195,56 +236,15 @@ impl LanguageServer for Backend {
             }
         }
 
-        if let Some(ref vcpkg_root) = initial
-            .root_uri
+        if let Some(ref project_root) = initial
+            .workspace_folders
             .as_ref()
-            .and_then(|uri| uri.to_file_path().ok())
+            .and_then(|folders| folders.first())
+            .and_then(|folder| folder.uri.to_file_path().ok())
         {
-            scansubs::scan_all(&vcpkg_root).await;
-            let mut root_path = self.root_path.lock().await;
-            root_path.replace(vcpkg_root.to_path_buf());
-
-            let build_dir = vcpkg_root.join("build");
-
-            if build_dir.is_dir() {
-                if let Some(query) = &*DEFAULT_QUERY {
-                    query.write_to_build_dir(build_dir.as_path()).ok();
-                }
-            }
-            if did_vcpkg_project(vcpkg_root) {
-                tracing::info!("This project is vcpkg project, start init vcpkg data");
-                let project_root = vcpkg_root;
-                let vcpkg_installed_path = project_root.join("vcpkg_installed");
-
-                #[cfg(unix)]
-                {
-                    use crate::utils::packagepkgconfig::QUERYSRULES;
-                    // When it is found to be a vcpkg project, the pc will be searched first from the vcpkg download directory.
-                    QUERYSRULES.lock().unwrap().insert(
-                        0,
-                        Box::leak(
-                            format!("{}/*.pc", vcpkg_installed_path.to_str().unwrap())
-                                .into_boxed_str(),
-                        ),
-                    );
-                }
-
-                // add vcpkg prefix
-                VCPKG_PREFIX.lock().unwrap().push(Box::leak(
-                    vcpkg_installed_path
-                        .to_str()
-                        .unwrap()
-                        .to_string()
-                        .into_boxed_str(),
-                ));
-
-                if let Ok(paths) = utils::make_vcpkg_package_search_path(&vcpkg_installed_path) {
-                    let mut vcpkg_libs = VCPKG_LIBS.lock().unwrap();
-                    for t in paths {
-                        vcpkg_libs.push(Box::leak(t.into_boxed_str()))
-                    }
-                }
-            }
+            self.root_path
+                .set(Some(project_root.to_path_buf()))
+                .expect("here should be the only place to set the root_path");
         }
 
         set_client_text_document(initial.capabilities.text_document);
@@ -256,6 +256,8 @@ impl LanguageServer for Backend {
                 version: Some(version),
             }),
             capabilities: ServerCapabilities {
+                rename_provider: Some(OneOf::Left(true)),
+                code_action_provider: Some(CodeActionProviderCapability::Simple(true)),
                 text_document_sync: Some(TextDocumentSyncCapability::Options(
                     TextDocumentSyncOptions {
                         open_close: Some(true),
@@ -361,10 +363,89 @@ impl LanguageServer for Backend {
         self.client
             .log_message(MessageType::INFO, "initialized!")
             .await;
+
+        let work_done_token = ProgressToken::Number(1);
+        let progress = self
+            .client
+            .progress(work_done_token, "start initing the workspace")
+            .with_message("initial start")
+            .with_percentage(0)
+            .begin()
+            .await;
+
+        if let Some(ref project_root) = self.root_path() {
+            progress
+                .report_with_message(&format!("start scanning {}", project_root.display()), 10)
+                .await;
+            scansubs::scan_all(&project_root, true).await;
+            let build_dir = project_root.join("build");
+            if build_dir.is_dir() {
+                if let Some(query) = &*DEFAULT_QUERY {
+                    query.write_to_build_dir(build_dir.as_path()).ok();
+                }
+            }
+            if did_vcpkg_project(project_root) {
+                progress
+                    .report_with_message("find vcpkg dir, start scanning", 20)
+                    .await;
+                tracing::info!("This project is vcpkg project, start init vcpkg data");
+                let vcpkg_installed_path = project_root.join("vcpkg_installed");
+
+                #[cfg(unix)]
+                {
+                    use crate::utils::packagepkgconfig::QUERYSRULES;
+                    // When it is found to be a vcpkg project, the pc will be searched first from the vcpkg download directory.
+                    QUERYSRULES.lock().unwrap().insert(
+                        0,
+                        Box::leak(
+                            format!("{}/*.pc", vcpkg_installed_path.to_str().unwrap())
+                                .into_boxed_str(),
+                        ),
+                    );
+                }
+
+                // add vcpkg prefix
+                VCPKG_PREFIX.lock().unwrap().push(Box::leak(
+                    vcpkg_installed_path
+                        .to_str()
+                        .unwrap()
+                        .to_string()
+                        .into_boxed_str(),
+                ));
+
+                if let Ok(paths) = utils::make_vcpkg_package_search_path(&vcpkg_installed_path) {
+                    let mut vcpkg_libs = VCPKG_LIBS.lock().unwrap();
+                    for t in paths {
+                        vcpkg_libs.push(Box::leak(t.into_boxed_str()))
+                    }
+                }
+            }
+        }
+        progress
+            .report_with_message("Start generating builtin commands", 50)
+            .await;
+        complete::init_builtin_command();
+        progress
+            .report_with_message("Start generating builtin module", 55)
+            .await;
+        complete::init_builtin_module();
+        progress
+            .report_with_message("Start generating builtin variable", 60)
+            .await;
+        complete::init_builtin_variable();
+        progress
+            .report_with_message("Start init system modules", 70)
+            .await;
+        complete::init_system_modules();
+        progress.report_with_message("Scan finished", 100).await;
+        progress.finish().await;
     }
 
     async fn shutdown(&self) -> Result<()> {
-        Ok(())
+        // NOTE: do nothing
+        // Seems tower_lsp won't do anything when receive this command.
+        // Now it should be proper for me to directly exit(0) here
+        exit(0)
     }
 
     async fn did_change_workspace_folders(&self, _: DidChangeWorkspaceFoldersParams) {
@@ -399,10 +480,10 @@ impl LanguageServer for Backend {
             if file_name.ends_with("txt") {
                 has_cached_changed = true;
                 if file_name == "CMakeLists.txt" {
-                    let Some(ref path) = *self.root_path.lock().await else {
+                    let Some(path) = self.root_path() else {
                         continue;
                     };
-                    scansubs::scan_all(path).await;
+                    scansubs::scan_all(path, false).await;
                     continue;
                 }
                 self.client
@@ -427,7 +508,7 @@ impl LanguageServer for Backend {
         let mut parse = Parser::new();
         parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
         let uri = input.text_document.uri.clone();
-        let context = input.text_document.text.clone();
+        let context = input.text_document.text;
         let mut storemap = BUFFERS_CACHE.lock().await;
         storemap.entry(uri.clone()).or_insert(context.clone());
         drop(storemap);
@@ -446,16 +527,38 @@ impl LanguageServer for Backend {
             uri,
             context,
             LintConfigInfo {
-                use_lint: self.init_info.lock().await.enable_lint,
+                use_lint: self.init_info().enable_lint,
                 use_extra_cmake_lint: true,
             },
         )
         .await;
+
         self.client
             .log_message(MessageType::INFO, "file opened!")
             .await;
     }
 
+    async fn code_action(&self, params: CodeActionParams) -> Result<Option<CodeActionResponse>> {
+        let Some(toolong) = params
+            .context
+            .diagnostics
+            .iter()
+            .find(|dia| dia.message.starts_with("[C0301]"))
+        else {
+            return Ok(None);
+        };
+
+        let uri = params.text_document.uri;
+        let storemap = BUFFERS_CACHE.lock().await;
+
+        let Some(context) = storemap.get(&uri) else {
+            return Ok(None);
+        };
+
+        let line = params.range.start.line;
+        Ok(quick_fix::lint_fix_action(context, line, toolong, uri))
+    }
+
     async fn did_change(&self, input: DidChangeTextDocumentParams) {
         // create a parse
         let uri = input.text_document.uri.clone();
@@ -468,14 +571,17 @@ impl LanguageServer for Backend {
                 uri,
                 context,
                 LintConfigInfo {
-                    use_lint: self.init_info.lock().await.enable_lint,
+                    use_lint: self.init_info().enable_lint,
                     use_extra_cmake_lint: false,
                 },
             )
             .await;
         }
         self.client
-            .log_message(MessageType::INFO, &format!("{input:?}"))
+            .log_message(
+                MessageType::INFO,
+                &format!("update file: {:?}", input.text_document.uri),
+            )
             .await;
     }
 
@@ -483,7 +589,7 @@ impl LanguageServer for Backend {
         let uri = params.text_document.uri;
         let storemap = BUFFERS_CACHE.lock().await;
 
-        let has_root = self.root_path.lock().await.is_some();
+        let has_root = self.root_path().is_some();
         let Some(context) = storemap.get(&uri).cloned() else {
             self.client
                 .log_message(MessageType::INFO, "file saved!")
@@ -499,7 +605,7 @@ impl LanguageServer for Backend {
             }
         };
         if has_root {
-            scansubs::scan_dir(&file_path).await;
+            scansubs::scan_dir(&file_path, false).await;
             complete::update_cache(&file_path, &context).await;
             jump::update_cache(&file_path, &context).await;
         }
@@ -507,8 +613,8 @@ impl LanguageServer for Backend {
             uri,
             context.to_string(),
             LintConfigInfo {
-                use_lint: self.init_info.lock().await.enable_lint,
-                use_extra_cmake_lint: true,
+                use_lint: self.init_info().enable_lint,
+                use_extra_cmake_lint: CMAKE_LINT_CONFIG.enable_external_cmake_lint,
             },
         )
         .await;
@@ -522,7 +628,6 @@ impl LanguageServer for Backend {
         let position = params.text_document_position_params.position;
         let uri = params.text_document_position_params.text_document.uri;
         let storemap = BUFFERS_CACHE.lock().await;
-        self.client.log_message(MessageType::INFO, "Hovered!").await;
         let Some(context) = storemap.get(&uri).cloned() else {
             return Ok(None);
         };
@@ -561,6 +666,7 @@ impl LanguageServer for Backend {
         let insert_final_newline = input.options.insert_final_newline.unwrap_or(false);
         match storemap.get(&uri) {
             Some(context) => Ok(getformat(
+                self.root_path().map(|p| p.as_path()),
                 context,
                 &self.client,
                 space_line,
@@ -579,8 +685,8 @@ impl LanguageServer for Backend {
                 format!("file {:?} closed!", params.text_document.uri),
             )
             .await;
-        //notify_send("file closed", Type::Info);
     }
+
     async fn completion(&self, input: CompletionParams) -> Result<Option<CompletionResponse>> {
         self.client.log_message(MessageType::INFO, "Complete").await;
         let location = input.text_document_position.position;
@@ -601,12 +707,13 @@ impl LanguageServer for Backend {
                 location,
                 &self.client,
                 &file_path,
-                self.init_info.lock().await.scan_cmake_in_package,
+                self.init_info().scan_cmake_in_package,
             )
             .await),
             None => Ok(None),
         }
     }
+
     async fn references(&self, input: ReferenceParams) -> Result<Option<Vec<Location>>> {
         let uri = input.text_document_position.text_document.uri;
         let location = input.text_document_position.position;
@@ -615,8 +722,6 @@ impl LanguageServer for Backend {
             return Ok(None);
         };
         drop(storemap);
-        let mut parse = Parser::new();
-        parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
         let file_path = match uri.to_file_path() {
             Ok(file_path) => file_path,
             Err(_) => {
@@ -624,8 +729,36 @@ impl LanguageServer for Backend {
                 return Err(LspError::internal_error());
             }
         };
-        Ok(jump::godef(location, context.as_str(), &file_path, &self.client, false).await)
+        Ok(jump::godef(
+            location,
+            context.as_str(),
+            &file_path,
+            &self.client,
+            false,
+            false,
+        )
+        .await)
     }
+
+    async fn rename(&self, input: RenameParams) -> Result<Option<WorkspaceEdit>> {
+        let edited = input.new_name;
+        let uri = input.text_document_position.text_document.uri;
+        let location = input.text_document_position.position;
+        let storemap = BUFFERS_CACHE.lock().await;
+        let Some(context) = storemap.get(&uri).cloned() else {
+            return Ok(None);
+        };
+        drop(storemap);
+        let file_path = match uri.to_file_path() {
+            Ok(file_path) => file_path,
+            Err(_) => {
+                tracing::error!("Cannot get file_path from {uri:?}");
+                return Err(LspError::internal_error());
+            }
+        };
+        Ok(rename::rename(&edited, location, file_path, &self.client, &context).await)
+    }
+
     async fn goto_definition(
         &self,
         input: GotoDefinitionParams,
@@ -651,7 +784,7 @@ impl LanguageServer for Backend {
                 return Err(LspError::internal_error());
             }
         };
-        match jump::godef(location, &context, &file_path, &self.client, true).await {
+        match jump::godef(location, &context, &file_path, &self.client, true, false).await {
             Some(range) => Ok(Some(GotoDefinitionResponse::Link({
                 range
                     .iter()
@@ -682,14 +815,13 @@ impl LanguageServer for Backend {
             None => Ok(None),
         }
     }
+
     async fn semantic_tokens_full(
         &self,
         params: SemanticTokensParams,
     ) -> Result<Option<SemanticTokensResult>> {
         let uri = params.text_document.uri.clone();
-        self.client
-            .log_message(MessageType::LOG, "semantic_token_full")
-            .await;
+
         let storemap = BUFFERS_CACHE.lock().await;
         match storemap.get(&uri) {
             Some(context) => Ok(semantic_token::semantic_token(&self.client, context).await),
